#include <stdio.h>
#include <string.h>
#include <ctype.h>

/* A vegyuletnev elso betujet nagybeture alakitjuk. Az elejen nem szamitanak a szammal vagy gorog betuvel irt helyzetszamok,
   a kiskapitalist jelzo ket makro, valamit a wiki es html dolt betus szoveg.

   A gorog betus helyzetszamokat szokas allonak es doltnek irni. Az allot felismerjuk, a dolt betus szoveget eleve
   figyelmen kivul hagyjuk.

   Ekezetes (utf-8) betut egyelore nem alakitunk nagybeture.

   Forras: Erdey-Gruz Tibor es Fodorne Csanyi Piroska: A magyar kemiai elnevezes es helyesiras szabalyai.
	Akademiai Kiado, Budapest, 1972., 1. kotet, 527. oldal.
*/

/* Szintaktikus egysegek, amiket az elotagban egy egysegkent kezelunk:
	- szam az ot esetleg koveto aposztrofokkal (ezt muszaj nezni, mert a kesobbieket megzavarja)
	- ''N'' es az ot esetleg koveto aposztrofok
	- <i>...</i> es ''...'' kozotti szoveg a hatarolokkal egyutt
	- mindenfele sablonhivas a parametereivel egyutt
	- gorog betuk es a prime
	- elvalaszto karakterek: - , ( )
*/

char *kezdobetu(char *vegyulet)		// visszaadja a nagybetu cimet
{
int i,valt,spn;
char *mut,*vegy = vegyulet;	// itt tartunk a feldolgozassal
static const char *fix[] = { "(+)-", "(−)-", "(±)-", "(", ")", "α", "β", "γ", "δ", "ε", "ζ", "′", ",", "-", },
	span[] = "<span ";

do	{
	for(valt=i=0; i < sizeof(fix)/sizeof(char *); i++)
		{	// atugorjuk a fix szovegeket
		spn = strlen(fix[i]);
		if(memcmp(vegy,fix[i],spn) == 0)
			{
			vegy += spn;
			valt = 1;
			break;
			}
		}
	if(valt)
		continue;
	if( (spn = strspn(vegy,"0123456789")) > 0)
		{	// szamjegy jott
		vegy += spn;
		vegy += strspn(vegy,"'");	// atugorjuk a koveto '-okat
		valt = 1;
		continue;
		}
	if(vegy[0] == '\'' && vegy[1] == '\'' && strchr("NO",vegy[2]) != NULL && vegy[3] == '\'' && vegy[0] == '\'')
		{	//''N'' vagy ''O'' alak, amit aposztrofok is kovethetnek
		vegy += 5;
		vegy += strspn(vegy,"'");	// atugorjuk a koveto '-okat
		valt = 1;
		continue;
		}
	if(vegy[0] == '\'' && vegy[1] == '\'' && vegy[2] != '\'')
		{	// dolt szoveg. Atugorjuk.
		vegy += 2;	// muszaj, kulonben vegtelen ciklusba eshetunk
		vegy += strcspn(vegy,"'");
		if(vegy[0] == '\'' && vegy[1] == '\'')
			{
			vegy += 2;
			valt = 1;
			continue;
			}
		else	break;		// nincs veguk a dolt betuknek
		}
	if(vegy[0] == '<' && vegy[1] == 'i' && vegy[2] == '>')
		{	// dolt szoveg <i> es </i> kozott. Atugorjuk.
		mut = strstr(vegy,"</i>");

		vegy = mut==NULL? vegy+3: mut+4;
		vegy += strspn(vegy,"'");	// <i>N</i>'
		valt = 1;
		continue;
		}
	if(strlen(vegy) > sizeof(span) && memcmp(vegy,span,sizeof(span)-1) == 0)
		{	// <span...> ... </span>. Kiskapitalisra hasznaljak.
		if( (mut = strstr(vegy,"</span>")) == NULL)
			{
			mut = strchr(vegy,'>');		// ha nincs vege a span-nek, legalabb a zaro kacsacsorig elmegyunk
			if(mut != NULL)
				mut++;
			else	mut = vegy + sizeof(span)-1;
			}
		else	mut += 7;	// 7 = </span> hossza
		vegy = mut;
		valt = 1;
		continue;
		}
	if(vegy[0] == '{' && vegy[1] == '{')
		{	// sablonhivas. Atugorjuk.
		vegy += 2;	// muszaj, kulonben vegtelen ciklusba eshetunk
		vegy += strcspn(vegy,"}");
		if(vegy[0] == '}' && vegy[1] == '}')
			{
			vegy += 2;
			valt = 1;
			continue;
			}
		else	break;		// nincs vege a sablonnak
		}
	if(vegy[0] == '[')
		{
		spn = strspn(vegy+1,".0123456789");
		if(vegy[1+spn] == ']')
			{
			vegy += spn+2;
			valt = 1;
			continue;
			}
		}
	}
while(valt && *vegy);

//puts(mut);		// teszt
if(islower(*vegy))
	*vegy = toupper(*vegy);
return vegy;
}

#ifdef	MAIN

//#include <stdio.h>
#include <libgyim.h>

int main(int argc, char *argv[])
{
char *mut, *tomb;
int ret;
//static const char apo[] =  "′";

//printf("%s hossza %d\n\n",apo,strlen(apo));
if(argc < 2)
	{
	fputs("Hivas: kezdobetu szoveg\n",stderr);
	return 1;
	}
tomb = memker(NULL,strlen(argv[1])+1);
strcpy(tomb,argv[1]);
//puts(tomb);
mut = kezdobetu(tomb);
ret = isupper(*mut);
//printf("haho: %c %d\n",*mut, ret);
if(mut > tomb)
	printf("%.*s   ",mut-tomb,tomb);
puts(mut);
free(tomb);
return !ret;
}

#endif
