/* okepl1: osszegkeplet feldolgozasa context free modon, stringen beluli helyettesitessel.
   Kulonfele alakokrol H2SO4 alakra hozzuk a kepletet. 
   Ez azert jo, mert a kulonfele alakok konnyen atalakithatok ra. Lehetnek benne ismetlodo vegyjelek es ()-ek.
*/

#include <string.h>
#include <ctype.h>
#include <libgyim.h>
#include "osszeg.h"

//#define	DEBUG	// teszt. Erdemes az egyik kimenetet faljba iranyitani, hogy a masik olvashato legyen.

static int helyett(char *txt, int mit, const char *mire);	// helyettesites helyben. A kozepso int nem tevedes.
static int szamhely(char *txt,int mit, const char *mire);	// helyettesites helyben szammal

static const char digit[] = "0123456789";

/* Az osszegkepletek feldolgozasanak elso menete. A bemenet rengetegfele lehet.
   A kimenetben csak vegyjelek, szamok es ()-ek lehetnek.
   Megengedett az 1-es atomszam, de el is hagyhato. Ugyanaz a vegyjel tobbszor is szerepelhet. */
	


int okepl1(char *txt)		// visszatereskor 0=siker, 10=kudarc: nem megengedett karakterek maradtak a konv. utan
{
/*	[: lasd Magnus's green salt
*/
//static const char ures[] = "", *chem[] = { "{{chem|", "{{Chem|" };
static const char ures[] = "", *chem[] = { "{{chem|", "chemical formula|" }, chem2[] = "{{chem2|",
		*chemtorl[] = { "|", "}}" };
char *mut;
int i;
struct tobbszovstr eredm;		// wikisabl.h

struct helystr {
	char *mit;
	const char *mire;
	};
static const struct helystr helytbl[] = { // fix szoveghelyettesitesek, beleertve a torlest is
		{ "<sub>", ures }, { "</sub>", ures }, { "'''", ures },
		{ "₀", "0" }, { "₁", "1" }, { "₂", "2" }, { "₃", "3" }, { "₄", "4" },
		{ "₅", "5" }, { "₆", "6" }, { "₇", "7" }, { "₈", "8" }, { "₉", "9" }
	};

/* A keplet sablonok (pl. {{carbon}} parameterei:
	1: jobb also index
	2: jobb felso index	- itt nem megengedett
	3: bal felso index	- itt nem megengedett
   A sablonbol csak a helyettesito ertek es esetleg a szam marad meg. */
static const struct helystr opcszam[] = {	// sablonok opcionalis (vagy kotelezo) szam parameterrel. 
		{ "{{sub", ures },	// kotelezo ugyan, de ezt nem ellenorizzuk
		{ "{{bromine", "Br" }, { "{{carbon", "C" },    { "{{chlorine", "Cl" },
		{ "{{fluorine", "F" }, { "{{hydrogen", "H" },  { "{{nitrogen", "N" },
		{ "{{oxygen", "O" },   { "{{platinum", "Pt" }, { "{{phosphorus", "P" }, { "{{sulfur", "S" },   { "{{s", "S" },
		{ "{{tin", "Sn" }
	};

#ifdef	DEBUG
fprintf(stderr,"%-30s  %s\n","az elejen:",txt);
#endif
for(mut=txt; (mut = strstr(mut,"{{")) != NULL; mut += 2)
	if(isupper(mut[2]))
		mut[2] = tolower(mut[2]);
#ifdef	DEBUG
fprintf(stderr,"%-30s  %s\n","kisbetus sablonnevek:",txt);
#endif
for(i=0; i < sizeof(helytbl)/sizeof(struct helystr); i++)	// fix helyettesites
	{
	mut = txt;
	while( (mut = strstr(mut,helytbl[i].mit)) != NULL)
		if(helyett(mut,strlen(helytbl[i].mit),helytbl[i].mire))
			mut++;	// sikertelen helyettesiteskor vegtelen ciklusba esnenk
	}
#ifdef	DEBUG
fprintf(stderr,"%-30s  %s\n","a fix helyettesitesek utan:",txt);
#endif
for(i=0; i < sizeof(opcszam)/sizeof(struct helystr); i++)	// {{sabl[|szam]}}
	{
	mut = txt;
	while( (mut = strstr(mut,opcszam[i].mit)) != NULL)
		{
		int mithossz = strlen(opcszam[i].mit);

		if(mut[mithossz] == '}' && mut[mithossz+1] == '}')
			{	// nincs szam
			if(helyett(mut,mithossz+2,opcszam[i].mire))
				mut++;
			}
		else	{
			if(mut[mithossz++] == '|')	// mithossz-ban most mar a | is benne van
				{
				if(szamhely(mut,mithossz,opcszam[i].mire))
					mut++;
				}
			}
		mut++;
		}
	}
#ifdef	DEBUG
fprintf(stderr,"%-30s  %s\n","[[ elott:",txt);
#endif
for(mut = txt; (mut = strstr(mut,"[[")) != NULL; )	// [[valami|vegyjel]] alak
	{
	int hossz1 = strcspn(mut,"|");
	int hossz2 = strcspn(mut,"]");
	int hossz  = strlen(mut);

	if(hossz1 >= hossz || hossz2 >= hossz || hossz1 >= hossz2 || mut[hossz2+1] != ']')	// nincs | es/vagy ]
		{
		mut++;
		continue;
		}
	// mut[hossz1]='|', mut[hossz2]=']' (a baloldali)
	memmove(mut,mut+hossz1+1,hossz-hossz1);	// toroltuk a [[..| reszt
	hossz2 -= hossz1+1;	// mut[hossz2]=']'
	memmove(mut+hossz2,mut+hossz2+2,hossz-hossz2-1);
	}
#ifdef	DEBUG
fprintf(stderr,"%-30s  %s\n","chem elott:",txt);
#endif
for(mut=txt; tobbszov(mut,sizeof(chem)/sizeof(char *),chem,&eredm) >= 0; )
	{
	char *m = eredm.vegcim;	// m {{chem| utanra mutat
	static const char *chemveg[] = { "}}", "{{" };

	if(tobbszov(eredm.vegcim,sizeof(chemveg)/sizeof(char *),chemveg,&eredm) !=  0)
		{	// vagy nincs lezaro }}, vagy chem-en belul masik sablon van
		mut += 2;
		continue;
		}
	for(i=0; *m && m < eredm.vegcim; ) // eredm.vegcim }} utanra mutat
		{	// ellenorizzuk, hogy minden masodik | utan szam all-e
		m += strcspn(m,"|");	// atugrottuk az akarmi lehet reszt
		if(m >= eredm.vegcim || *m != '|')
			break;	// nem jon ujabb parameter
		if(isupper(*++m))	// ha vegyjel jon, ujra kezdjuk a vizsgalatot
			continue;
		m += strspn(m,digit);	// atleptuk a szamjegyeket
		if(*m == '}')
			break;	// a szam utan nincs tobb parameter
		if(*m == '|')
			{	// minden rendben, a szam utan ujabb parameter jon
			m++;
			continue;
			}
		i = -1;		// a szam utan rossz dolgok jonnek
		break;
		}
	if(i)	{
		mut = eredm.vegcim;
		continue;
		}
	m = strchr(mut,'|')+1;		// minden rendben. Lehet csinalni.
	memmove(mut,m,strlen(m)+1);	// toroljuk {{chem|-et
	while(tobbszov(mut,sizeof(chemtorl)/sizeof(char *),chemtorl,&eredm) == 0)	// toroljuk |-eket
		memmove(eredm.kcim,eredm.vegcim,strlen(eredm.vegcim)+1);
	memmove(eredm.kcim,eredm.vegcim,strlen(eredm.vegcim)+1);	// toroljuk }}-t
	}
#ifdef	DEBUG
fprintf(stderr,"%-30s  %s\n","chem2 elott:",txt);
#endif
for(mut=txt; (mut = strstr(mut,chem2)) != NULL; )
	{
	char *m = mut + strlen(chem2);

	if(tobbszov(m,sizeof(chemtorl)/sizeof(char *),chemtorl,&eredm) == 1)
		{	// chem2-nek egy parametere van
		memmove(mut,m,strlen(m)+1);	// {{chem2| torlese
		m = strstr(mut,"}}");
		memmove(m,m+2,strlen(m+2)+1);
		}
	else	mut++;
	}

#ifdef	DEBUG
fprintf(stderr,"%-30s  %s\n","kilepeskor:",txt);
#endif
for(mut=txt; *mut ; mut++)
	if(!isalpha(*mut) && !isdigit(*mut) && *mut != '(' && *mut != ')')
		return 10;
return 0;
}


	// helyettesites  helyben, a string elejen. "mire" rovidebb kell legyen, mint "mit"
static int helyett(char *txt, int mit, const char *mire)	 //A kozepso int nem tevedes.
{
int hossz = strlen(mire);

if(mit < 1 || hossz > mit)
	return 11;	// kulonben a hivo vegtelen ciklusba eshetnek
memcpy(txt,mire,hossz);
memmove(txt+hossz,txt+mit,strlen(txt)-mit+1);
return 0;
}


static int szamhely(char *txt,int mit, const char *mire)	// helyettesites helyben szammal
{	//  mit a {{ es | kozotti resz hossza
char mivel[10];		// ide tesszuk a vegyjel szam stringet
int mivelhossz  = strlen(mire);
int szamhossz = strspn(txt+mit,digit);

strcpy(mivel,mire);	// vegyjel
if(mivelhossz + szamhossz < sizeof(mivel))
	{	// belefer, aminek kell
	memcpy(mivel+mivelhossz,txt+mit,szamhossz);	// a szam is benne van mivel-ben
	mivel[mivelhossz+szamhossz] = 0;	// mivel kesz
	mit += szamhossz;
	if(txt[mit] == '}' && txt[mit+1] == '}' && !helyett(txt,mit+2,mivel) )
		return 0;
	}
return 11;
}


#ifdef	MAIN

#include <stdlib.h>
#include <libgyim.h>

int main(int argc,char *argv[])
{
char *mut;
int i,tst,hossz;
static const char *tstdat[] = {	// tisztit-ot okepl hivja meg, nem okepl1
	"HCN",					// egyszeru
	"H'''C'''N",				// vastagitas
//	"HCN<ref>valami</ref>",			// ref
//	"HCN<ref name=\"x\"/>",			// />-re vegzodo ref
//	"HC<!--x-->N",				// html komment
	"H<sub>2</sub>SiO<sub>3</sub>",		// sub
	"C₈H₁₀AsNO₅",				// wikidata
	"C<sub>6</sub>H<sub>5</sub>NH(COCH<sub>3</sub>)",
	"{{carbon}}<sub>27</sub>{{hydrogen}}<sub>48</sub>{{oxygen}}",	// opcionalis szammal es szam nelkul
	"C{{sub|26}}H{{sub|24}}P{{sub|2}}"	// kotelezo szam
	"[[Carbon|C]]<sub>48</sub>[[Hydrogen|H]]<sub>72</sub>[[Oxygen|O]]<sub>14</sub>"	// [[
	"{{Chem|TlH|3|O|3}}",
	"{{chem|C|6|H|4|(COOH)|2}}",
	};

//szoveg = "C<sub>6</sub>H<sub>5</sub>NH(COCH<sub>3</sub>)";
if(argc < 2)
	{
	fputs("Hivas: okeplc { osszegkeplet | @ }. Pl.:\n./sub 'C<sub>6</sub>H<sub>5</sub>NH(COCH<sub>3</sub>)'\n",stderr);
	exit(1);
	}
tst = !strcmp(argv[1],"@");
if(tst)
	{	// munkatomb lefoglalasa
	for(hossz=i=0; i < sizeof(tstdat)/sizeof(char *); i++)
		{
		int h = strlen(tstdat[i]);

		if(h > hossz)
			hossz = h;
		}
	}
else	hossz = strlen(argv[1]);
mut = memker(NULL,hossz+1);

if(tst) for(i=0; i < sizeof(tstdat)/sizeof(char *); i++)
	{
	int fmt = hossz+5;	// ilyen szeles a tesztadat a kiiraskor

	strcpy(mut,tstdat[i]);
	printf("%-*s",fmt,mut);
	okepl1(mut);
	okepl1ki(mut);
	puts("");
	if(!tst)
		break;
	}
else	{	// parameterbol kapott tesztadatra reszletesebb a kimenet
	strcpy(mut,argv[1]);
	puts(mut);
	tisztit(mut);
	printf("\ttisztit: %s\n",mut);
	i = okepl1(mut);
	printf("\tokepl1  %d  %s\n",i,mut);
	okepl1ki(mut);
	puts("");
	}
return 0;
}

#endif
