/* Az osszegkeplet feldolgozas masodik, context-sensitive resze. A bemenet alakja:

<osszegkeplet>  := <atomcsoport> | ( <atomcsoport> ) <darabszam>
<atomcsoport>   := <atom> | <atom> <darabszam> | <atomcsoport>...
<atom>          := <vegyjel> | [[szoveg|<vegyjel]] | {{vegyjel}}
<darabszam>     := <sub> <szam> </sub>
<vegyjel>       := <nagybetu> | <nagybetu> <kisbetu>

Egy vegyjel tobbszor szerepelhet: a program osszegzi az elofordulasokat.
A bemeno sor mar nem valtozik, az eredmeny a hivo teruleten, szimboltablaban keletkezik.
*/

#include <stdlib.h>
#include <search.h>
#include <string.h>
#include <ctype.h>
#include "osszeg.h"

#define	mutkonv(mut)		((struct elemstr *)mut)->vjel

static const char *sor;


static int rekur(struct keplstr *);
static int rendez(const void *egyik,const void *masik);		// C, H, abc-rend
static int rendez2(const void *egyik,const void *masik);	// abc-rend
static int hasonl(const void *egyik,const void *masik)
{
return strcmp(mutkonv(egyik),mutkonv(masik));
}


int okepl2(const char *sorbe,struct keplstr *eredm)
{
sor = sorbe;
eredm->eszam = 0;
return rekur(eredm);
}


static int rekur(struct keplstr *eredm)
{
struct keplstr keplet;
struct elemstr elem,*emut;
size_t szam;
int k,valt,db;
static const char digit[] = "0123456789";

do	{
	valt = 0;
	if(isupper(*sor))
		{
		if(eredm->eszam >= ESZAM-1)
			return 20;		// nincs tobb hely a szimboltablaban
		memset(&elem,0,sizeof(elem));
		elem.vjel[0] = *sor++;
		if(islower(*sor))
			elem.vjel[1] = *sor++;
		if(!vjelkeres(elem.vjel))
			return 22;		// ismeretlen vegyjel
		if(sscanf(sor,"%d",&elem.darab) == 1)
			sor += strspn(sor,digit);
		else	elem.darab = 1;
				// konvertaltunk egy elemet. Bele kell tenni az eredmenytombbe.
		szam = eredm->eszam;	// innen tudjuk, hogy benne volt-e a tablaban, vagy most tette bele
		emut = lsearch(&elem,eredm->elem,&szam,sizeof(struct elemstr),hasonl);
		if(szam > eredm->eszam)	// igaz, ha lsearch uj erteket tett bele
			eredm->eszam = szam;
		else	emut->darab += elem.darab;
		valt++;
		}
	if(*sor == ')' || *sor == 0)
		return 0;
	if(*sor == '(')
		{
		sor++;
		keplet.eszam = 0;
		if(rekur(&keplet) || *sor != ')')
			return 21;		// rekur ismeretlen karakter miatt tert vissza
		if(sscanf(++sor,"%d",&db) != 1)
			db = 1;
		else	sor += strspn(sor,digit);
		for(k=0; k < keplet.eszam; k++)		// osszefesuljuk a visszakapott eredmenyt a mienkkel
			{
			keplet.elem[k].darab *= db;
			if(eredm->eszam >= ESZAM-1)
				return 20;		// nincs tobb hely a szimboltablaban
			szam = eredm->eszam;	// innen tudjuk, hogy benne volt-e a tablaban, vagy most tette bele
			emut = lsearch(keplet.elem+k,eredm->elem,&szam,sizeof(struct elemstr),hasonl);
			if(szam > eredm->eszam)	// nem volt benne
				eredm->eszam = szam;
			else	emut->darab += keplet.elem[k].darab;
			}
		valt = 1;
		}
	} while(valt);
return 21;	// ismeretlen karakternel tartunk
}


void okeplki(char *tomb,struct keplstr  *eredm)
{	// tomb merete 8*elemszam+1, ha egy elem max. 9999-szer szerepel
int k;

for(tomb[0]=k=0; k < eredm->eszam; k++)
	sprintf(tomb+strlen(tomb),"|%s=%d",eredm->elem[k].vjel,eredm->elem[k].darab);
}


static int rendez(const void *egyik,const void *masik)		// C, H, abc-rend
{
int ret = strcmp(mutkonv(egyik),mutkonv(masik));
static char C[] = "C", H[] = "H";

if(ret)	{	// a ket  vegyjel nem egyforma
	if(strcmp(mutkonv(egyik),C) == 0)
		return -1;
	if(strcmp(C,mutkonv(masik)) == 0)	// C mar egyik sem lehet
		return 1;
	if(strcmp(mutkonv(egyik),H) == 0)
		return -1;
	if(strcmp(H,mutkonv(masik)) == 0)	// H mar egyik sem lehet
		return 1;
	}
return ret;
}

static int rendez2(const void *egyik,const void *masik)		// abc-rend
{
return strcmp(mutkonv(egyik),mutkonv(masik));
}

void keplrend(struct keplstr *kepl)
{	// C, H, abc rendbe keplrendi a keplet vegyjeleit
qsort(kepl->elem,kepl->eszam,sizeof(struct elemstr),rendez);
}

void keplrend2(struct keplstr *kepl)
{	// C, H, abc rendbe keplrendi a keplet vegyjeleit, ha van C, egyebkent abc rendbe
keplrend(kepl);
if(strcmp(kepl->elem[0].vjel,"H") == 0)	// nincs szen: nem kell elorevenni H-t
	qsort(kepl->elem,kepl->eszam,sizeof(struct elemstr),rendez2);
}


/* Kiszamolja, hany byte kell a kiiro tombnek. Egy elem |Xx=n... alaku, plusz a vegen a \0.
   Vissza: INT_MAX, ha tul nagy a molekula (100000-nel tobbszor fordul elo egy elem).
*/

#include <limits.h>

int keplkihossz(struct keplstr *keplet)		// milyen hosszu puffer kell keplet kiirasahoz
{
static const int digszam[] = { 10, 100, 1000, 10000, 100000 };
int i,k,s;

for(i=0,s=1; i < keplet->eszam; i++)
	{
	for(k=0; k < sizeof(digszam)/sizeof(int); k++)
		if(keplet->elem[i].darab < digszam[k])
			break;
	s += k < sizeof(digszam)/sizeof(int)? 2 + strlen(keplet->elem[i].vjel) + k+1 : INT_MAX;
	}
return s;
}

#ifdef	MAIN

#include <stdio.h>

static void tstkiir(const char *str);

int main(int argc,char *argv[])
{
int i;
static const char *tstdat[] = {		// beepitett tesztadatok
	"Na2SO4",
	"CH3COOH",
	"Ca5(PO4)3OH",	// apatit
	"CH4(OH)2",	// pl. rezorcin
	};

//szoveg = "C<sub>6</sub>H<sub>5</sub>NH(COCH<sub>3</sub>)";
if(argc < 2)
	{
	fputs("Hivas: okepl2 { osszegkeplet | @ }. Pl.:\n./sub 'C6H5NH(COCH3)'\n",stderr);
	exit(1);
	}

if(strcmp(argv[1],"@") == 0)
	{
	for(i=0; i < sizeof(tstdat)/sizeof(char *); i++)
		tstkiir(tstdat[i]);
	}
else	tstkiir(argv[1]);
return 0;
}

static void tstkiir(const char *str)
{
struct keplstr eredm;
#define	THOSSZ	(8*ESZAM+1)
char ki1[THOSSZ],ki2[THOSSZ];
int ret,hossz;

ret = okepl2(str,&eredm);
hossz = keplkihossz(&eredm);
printf("%2d %3d   %-30s",ret,hossz,str);
if(ret)
	printf("Maradek  sor: %s\n",sor);
if(hossz > THOSSZ)
	printf("Tul nagy a molekula: ki sem tudom irni\n");
else	{
	okeplki(ki1,&eredm);
	keplrend(&eredm);
	okeplki(ki2,&eredm);
	printf(" %-30s %-30s\n",ki1,ki2);
	}
}

#endif
