#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <wikisabl.h>
#include "osszeg.h"


FILE *fpsmiles;	// smiles
static char aliasnev[256] = "",
	sabnev[256] = "",	// sablonnev
	parnev[256] = "",	// parameternev
	keplet[256]  = "";	// osszegkeplet |C=10|H=18|O=1 alakban
static int hibaflg;	// 1: az osszegkepletet ket kulonbozo modon adtak meg


static void masol(char *dest,const char *sour,int maxmeret);
static int hasonl(const char *egyik,const char *masik);	// osszehasonlit ket osszegkepletet. 1 = egyformak.


// Kemiai sablonok osszegkeplet parameterei
static const char *okeplnev[] = { "Formula", "[[Képlet]]", "kémiai_képlet", "chemical_formula", "Képlet", "képlet" };
		// Es ki kell irni a vegyjeleket: egy nagybetu, ami utan johet egy kicsi
		// Es ki kell irni a chem sablon osszes parameteret

#if	0
Az osszegkeplet tobb sablonban es kulonfele parameterekben adhato meg.
A cel az, hogy akkor es csak akkor irjuk ki, ha valamennyi ertek azonos.
Mivel a parszer sablononkent hivja sablfv-t, a szocikkvaltasrol a foprogramon keresztul kell ertesulni.

Az osszegkeplet nemcsak egy-egy parameterben adhato meg, hanem vegyjelekkel is, 
amit csak az osszes parameter vegignezese utan ismerunk meg.
Ezert addig nem irunk ki semmit, es a vegyjelekkel keszulo osszegkepletet mashol tartjuk.
Eloirjuk, hogy a vegyjelek ugyanabban a sablonban legyenek.

Algoritmus:

Ha uj szocikk: keplet felejt.
Ha uj sablon: vjelkepl felejt.
Vegigmegyunk a parametereken:
	- ha vegyjelet talalunk, hozzairjuk vjelkepl-hez
	- ha osszegkepletet talalunk:
		-- meg nincs keplet: most mar van (keplet[0]-ban jelezzuk)
		-- ha mar volt keplet, ami azonos a most talalttal: nem csinaluk semmit
		-- ha mar volt keplet, ami nem azonos a most talalttal: hibajelzes (hibaflg=1)
Ha talaltunk vjelkepletet, van mar osszegkeplet, es a ketto nem azonos: hibajelzes
Ha talaltunk vjelkepletet, es meg nincs osszegkeplet, akkor mar van.

Az osszegkepletet a szocikk-lezaro fuggvenyben irjuk ki.

Ket keplet azonossagakor stringegyezest nezunk: a kulonbozo sorrendben megadott atomszamokat kulonbozoeknek tekintjuk. Ez konnyen valtoztathato.
#endif

void sablfv(struct sablstr sabltomb, int debug)
{
int i, par, vegyflg = !strcmp(sabltomb.nev,"Vegyület_infobox"),	// a sablonnev Vegyület_infobox?
	sabltip = kemsabl(sabltomb.nev);	// 0=nem kemiai, 1=alsablon, 2=fosablon
char *pnev,	// parameternev
	*ertek;	// ertek
char *mut,vjelkepl[80];		// eredmeny |C=10|H=18|O=1 alakban

if(!sabltip)
	return;		// nem kemiai sablon
for(vjelkepl[0]=par=0; par < sabltomb.parszam; par++)
	{
	ertek = sabltomb.param[par].ertek;
	if(!ertek[0])	// a parameter erteke ures
		continue;
	pnev = sabltomb.param[par].parnev;
	if(vjelkeres(pnev))
		{	// vegyjel: betesszuk vjelkepl-be
		if(vegyflg && !pnev[1] && strchr("HFRO",*pnev) == NULL)		// nem lenne szabad vegyjelparameternek lennie. Logoljuk
			fprintf(stderr,"* %s: Vegyjel Vegyület_infoboxban: %s\n",szocikk,pnev);
		if(!vegyflg || pnev[1] || strchr("HFRO",*pnev) == NULL)
			{		// Vegyulet infoboxban a negy parameter nem osszegkepletet jelent
			i = 0;
			if(sscanf(ertek,"%d",&i) && i > 0 && i < 99999)
				{	// As=8 alaku vegyjel. |As=99999 hossza max. 8 byte.
				if( (mut = vjelkepl + strlen(vjelkepl)) < vjelkepl + sizeof(vjelkepl) - 8)	// remelhetoleg mindig teljesul
					sprintf(mut,"|%s=%d",pnev,i);
				}
			}
		}
	else	{
		//printf("Ertek kiirasa: %s\n",ertek);
		for(i=0; i < sizeof(okeplnev)/sizeof(char *); i++)
			if(strcmp(okeplnev[i],pnev) == 0)
				{	// osszegkeplet a Formula (vagy hasonlo) parameterben
				char eredm[sizeof(keplet)];
				int okeplret = okepl(ertek,0,eredm,sizeof(eredm));

				if(okeplret == 0)
					{	// az erteket sikerult konvertalni
					if(keplet[0])
						{
						i = hasonl(keplet,eredm);	// i = 0: egyforma a ket keplet
						fprintf(stderr,
							"%s%s: ket osszegkeplet. %s = %s, %s = %s, a ketto %s azonos\n",
							i? "*** ": "",szocikk, sabnev,keplet, sabltomb.nev,eredm,
							i? "nem": "");
						if(i)
							hibaflg = 1;
						}
					else	{
						strcpy(keplet,eredm);
						strcpy(sabnev,sabltomb.nev);
						masol(parnev,pnev,sizeof(parnev));
						}
					}
				else	fprintf(stderr,"*** %s: Hibas osszegkeplet: %s\n",szocikk,ertek);
				//break;
				}
		// Az osszegkeplet-parametereket feldolgoztuk. A tobbi jon.
		if(fpsmiles != NULL && (strcmp(pnev,"SMILES")==0 || strcmp(pnev,"smiles")==0) )
			{
			tisztit2(ertek);	// kommentek es uj sorok elhagyasa
			if(*ertek)
				fprintf(fpsmiles,"%s|%s\n",szocikk,ertek);	// smiles
			}
		if(strcmp(pnev,"Name")==0 || strcmp(pnev,"név")==0 || strcmp(pnev,"drug_name") == 0)
			{
			tisztit2(ertek);	// ref-ek, html kommentek, uj sorok elhagyasa
			tisztithash(ertek);	// a #-jeleket kiszedjuk
			//masol(aliasnev,kanname(ertek),sizeof(aliasnev));
			masol(aliasnev,ertek,sizeof(aliasnev));
			}
		}
	}
if(!hibaflg && keplet[0] && vjelkepl[0])
	{	// osszegkeplettel es vegyjelekkel is megadtak
	if(hasonl(keplet,vjelkepl))
		{	// a ketto kulonbozik
		fprintf(stderr,"***%s: keplettel es vegyjelekkel elteroen megadott osszegkeplet. %s = %s, vegyjelekkel: %s\n",
			szocikk, sabnev,keplet, vjelkepl);
		hibaflg = 1;
		}
	}
if(!hibaflg && !*keplet && vjelkepl[0])
	{	// ha a kepletet vegyjelekkel adtak meg, odamasoljuk a formula helyere
	masol(keplet,vjelkepl,sizeof(keplet));
	masol(sabnev,sabltomb.nev,sizeof(sabnev));	// parnev-et  mar kitoltottuk
	i = strlen(sabnev);
	masol(sabnev+i," (vegyjelekkel)",sizeof(sabnev)-i);
	}
if(sabltip == 2)	// fosablon vege. Kiirunk.
	{
	if(*aliasnev == 0)
		{
		strcpy(aliasnev,szocikk);
		for(mut=aliasnev; (mut = strchr(mut,'_')) != NULL; *mut = ' ')
			;
		}
	
	printf("%s#%s",szocikk,aliasnev);
	if(!hibaflg && *keplet == '|')
		printf("#%s",keplet+1);
	puts("");
	ujcikk();
	}
}



static void masol(char *dest,const char *sour,int maxmeret)
{
while(--maxmeret > 0 && *sour)		// strncpy?
	*dest++ = *sour++;
*dest = 0;
}


static int hasonl(const char *egyik,const char *masik)		// osszehasonlit ket osszegkepletet
{	// vissza: 0 = egyforma es mindketto hibatlan
char be[sizeof(keplet)], ki1[sizeof(keplet)],ki2[sizeof(keplet)];
int ret;

okeplbe(be,egyik);
ret = okepl(be,1,ki1,sizeof(ki1));
okeplbe(be,masik);
return ret || okepl(be,1,ki2,sizeof(ki2)) || strcmp(ki1,ki2);
}


void ujcikk()
{
*keplet = *aliasnev = hibaflg = 0;
}

void cikkveg() {}
